function xs = ams_mvnewtons(fpnt, x0, pars, tol, alpha, maxiter, dx, verbose)
%Multi variable newton's method solver
%Parameters
%fpnt - function pointer taking fpnt(x,pars) and returning function vector
%x0 - initial guess (column vector) for x which solves fpnt
%pars - parameter vector passed to fpnt
%tol - relative tolerance for the norm of residual (euclidean norm - scale
%accordingly)
%alpha - underrelaxation factor to the algorithm - damps divergence
%maxiter - maximum iterations
%dx - scale over which derivative is taken
%verbose - vocalize warnings

%set defaults
if(nargin < 8) verbose = 0; end
if(nargin < 7) dx = 1E-8; end
if(nargin < 6) maxiter = 100000; end
if(nargin < 5) alpha = 0.5; end
if(nargin < 4) tol = 0.001; end
if(nargin < 3) pars = []; end

dettol = 1E-6;

xs = x0;

N = size(x0,1); %size of the x vector;

I = 1;
exitflag = 0;
while(I<maxiter && exitflag == 0)
    xsold = xs;
    
    %Jacobian matrix
    dfdx = zeros(N,N);
    for K = 1:N
        q1 = (fpnt(xs+dx.*ovect(N,K),pars)-fpnt(xs-dx.*ovect(N,K),pars))/(2.0*dx);
        for J = 1:N
            dfdx(J,K) = q1(J);
        end
    end
    fp = fpnt(xs,pars);
    %case - derivative equals zero at this location
    if(abs(det(dfdx)) <= dettol)
        if(verbose==1)
            fprintf('Encountered local extremum, xs = %1.6g, iter = %d - moving randomly\n',xs,I);
        end
        %randomly move within 10 dx to get to a spot that
        %doesn't suck
        xs = xs + 10.0.*(2.0.*rand(N,1)-1.0).*dx;
    else
        %calculate newton's step
        ns = -inv(dfdx)*fp;
        %update position
        xs = xs + alpha*ns;
        
        rerr = norm(xs-xsold);
        if(rerr<tol)
            exitflag = 1;
            if(verbose==1)
                fprintf('Success, root found, iteration %d\n',I);
            end
        end
    end

    I = I + 1;
end

if(I>=maxiter)
    if(verbose==1)
        fprintf('Failure: Maximum iterations exceeded\n');
    end
end

return

function ov = ovect(N,I)
    ov = zeros(N,1);
    ov(I) = 1;
return