%Aaron M. Schinder
%9 Aug 2015
%Involute gear curve calculator
%
%Inputs:
%diampitch - Diametrical Pitch (# of teeth for 1 unit diameter gear)
%nteeth - (# of teeth for this gear)
%phi_pitch - gear pitch angle
%addp - addendum / circular pitch (standard is 1.00/pi)
%           (for stub teeth, it is 1.6/2 * that);
%dedp - dedendum / circular pitch (standard is 1.25/pi)
%
%Outputs:
%xgl,ygl - a list of x and y coordinates for the outer edge of a spur gear
%rgl,thgl - a list of r and theta coordinates for the outer edge of a spur gear
%ccs - a structure containing calculated properties of the gear, such as
%	ccs.ro - outer radius
%	ccs.rb - base radius
%	ccs.rr - root radius
%	ccs.rp - pitch radius
%	ccs.a - addendum
%	ccs.b - dedendum
%	ccs.dao - angular length between the involute base and top of the tooth
%	ccs.dap - angular length between the involute base and pitch circle
%	ccs.dar - angular length between the involute base and root
%...etc
function [xgl,ygl,rgl,thgl,ccs] = calculate_gear(diampitch,nteeth,phi_pitch,addp,dedp)

p = pi/diampitch;
a = addp*p;
b = dedp*p;

rp = 1.0*nteeth/diampitch/2;  %pitch radiaus
rb = rp*cos(phi_pitch);       %base radius
rr = rp-b;                    %root radius
ro = rp + a;                  %outer radius

xgl = [];
ygl = [];
rgl = [];
thgl = [];
ccs = struct();
ccs.rp = rp;
ccs.rb = rb;
ccs.rr = rr;
ccs.ro = ro;
ccs.a = a;
ccs.b = b;

%calculate angles

dto = sqrt(ro^2/rb^2-1);
dtp = sqrt(rp^2/rb^2-1);
if(rr^2/rb^2>1)
    dtr = sqrt(rr^2/rb^2-1);
else
    dtr = 0;
end

dao = inv_alp_theta(dto);
dap = inv_alp_theta(dtp);
dar = inv_alp_theta(dtr);

ccs.dao = dao;
ccs.dap = dap;
ccs.dar = dar;

if(dao>pi/2/nteeth);
    %if the tooth must be too narrow for the outer diameter, give warning
    fprintf('Warning. da0 = %1.6g deg, when only %1.6g deg are available for tooth tip\n',dao*180*pi,90/nteeth);
    %calculate new outer diameter
    dao = pi/2/nteeth;
    [xx,yy] = inv_alp(dao);

    ro = rb*sqrt(xx*xx+yy*yy);
end
    
Nlnd = 4;
Ninv = 4;

I = 1;
for Sect = 1:nteeth
    a0 = (Sect-1)*2*pi/nteeth;
    
    %first sector: outer
    amn = 0;
    amx = pi/2/nteeth-dao+dap;
    da = (amx-amn)/(Nlnd-1);
    a = amn;
    for J = 1:Nlnd
        rgl(I) = ro;
        thgl(I) = a+a0;
        a = a + da;
        I = I + 1;
    end
    
    %second sector - the involute curve
    amn = pi/2/nteeth-dao+dap;
    amx = pi/2/nteeth;
    da = (amx-amn)/Ninv;
    a = amn+da;
    for J = 1:Nlnd
        q = dap-(a-pi/2/nteeth);
        rgl(I) = rb*inv_alp(q);
        thgl(I) = a+a0;
        
        %fprintf('%1.6g,%1.6g,%1.6g,%1.6g\n',a,q,inv_alp(q),rgl(I));
        
        a = a + da;
        I = I + 1;
    end
    
    %third sector (root)
    amn = pi/2/nteeth;
    amx = 3*pi/2/nteeth;
    da = (amx-amn)/(Nlnd-1);
    a = amn;
    for J = 1:Nlnd
        rgl(I) = rr;
        thgl(I) = a+a0;
        a = a + da;
        I = I + 1;
    end
    
    a = a-da;
    
    %fourth sector: second involute
    amn = 3*pi/2/nteeth;
    amx = 3*pi/2/nteeth+dao-dap;
    da = (amx-amn)/Nlnd;
    a = amn;
    for J = 1:Nlnd
        q = dap+(a-3*pi/2/nteeth);
        rgl(I) = rb*inv_alp(q);
        thgl(I) = a+a0;
        a = a + da;
        I = I + 1;
    end
    
    
    %fifth sector: outer curve
    amn = 3*pi/2/nteeth+dao-dap;
    amx = 2*pi/nteeth;
    da = (amx-amn)/Nlnd;
    a = amn;
    for J = 2:Nlnd
        rgl(I) = ro;
        thgl(I) = a+a0;
        a = a + da;
        I = I + 1;
    end
    
end

%calculate x and y

for I = 1:size(rgl,2)
    xgl(I) = rgl(I)*cos(thgl(I));
    ygl(I) = rgl(I)*sin(thgl(I));
end


return
